package com.hydrogen.mbg.utils;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;
//import io.jsonwebtoken.Claims;
//import io.jsonwebtoken.Jwts;
//import io.jsonwebtoken.SignatureAlgorithm;
//import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * JwtToken生成的工具类
 * JWT token的格式：header.payload.signature
 * header的格式（算法、token的类型）：
 * {"alg": "HS512","typ": "JWT"}
 * payload的格式（用户名、创建时间、生成时间）：
 * {"sub":"wang","created":1489079981393,"exp":1489684781}
 * signature的生成算法：
 * HMACSHA512(base64UrlEncode(header) + "." +base64UrlEncode(payload),secret)
 */
@Component
public class JwtTokenUtil {
    private static final Logger LOGGER = LoggerFactory.getLogger(JwtTokenUtil.class);
    private static final String CLAIM_KEY_USERNAME = "account";
    private static final String CLAIM_KEY_USERID = "account_id";
    private static final String CLAIM_KEY_CREATED = "created";
    private static final String ISSUER = "hydrogen";
    @Value("${jwt.secret}")
    private String secret;
    @Value("${jwt.expiration}")
    private Long expiration;
    @Value("${jwt.tokenHead}")
    private String tokenHead;

    // 创建Token
    public String createToken(String userName,String userId) {
        return tokenHead + JWT.create()
                .withIssuer(ISSUER)  //发行人
                .withClaim(CLAIM_KEY_USERNAME,userName) //存放数据
                .withClaim(CLAIM_KEY_USERID,userId) //存放数据
                .withClaim(CLAIM_KEY_CREATED,new Date()) //存放数据
//                .withExpiresAt(new Date(System.currentTimeMillis() + expiration * 1000)) //过期时间,Redis处理
                .sign(Algorithm.HMAC256(secret));
    }
//
    // 从Token中获取用户信息
    public Map<String,String> getUserInfoFromToken(String token) {
        JWTVerifier jwtVerifier = JWT.require(Algorithm.HMAC256(secret))
                .withIssuer(ISSUER)
                .build();
        token = token.replace(tokenHead,"");
        DecodedJWT decodedJWT = jwtVerifier.verify(token);
        Map<String,String> userInfo = new HashMap<>();
        userInfo.put(CLAIM_KEY_USERNAME,decodedJWT.getClaim(CLAIM_KEY_USERNAME).asString());
        userInfo.put(CLAIM_KEY_USERID,decodedJWT.getClaim(CLAIM_KEY_USERID).asString());
        return userInfo;
    }

    // 验证Token是否有效
    public boolean validateToken(String token) {
        boolean isValid = true;
        JWTVerifier jwtVerifier = JWT.require(Algorithm.HMAC256(secret))
                .withIssuer(ISSUER)
                .build();
        try {
            token = token.replace(tokenHead,"");
            DecodedJWT decodedJWT = jwtVerifier.verify(token);
        }catch(Exception e){
            isValid = false;
            LOGGER.info("token异常："+e);;
        }
        return isValid;
    }



    public static void main(String[] args) {
        Map<String, Object> claims = new HashMap<>();
//        claims.put(CLAIM_KEY_USERNAME, "fujichuan");
        claims.put(CLAIM_KEY_CREATED, new Date());
        String token = JWT.create()
                .withIssuer("auth0")  //发行人
                .withClaim("username","fujichuan") //存放数据
                .withClaim("created",new Date())
                .withExpiresAt(new Date(System.currentTimeMillis() + 604800 * 1000)) //过期时间
                .sign(Algorithm.HMAC256("userSecret"));


        JWTVerifier jwtVerifier = JWT.require(Algorithm.HMAC256("userSecret"))
                .withIssuer("auth0")
                .build();
        DecodedJWT decodedJWT = jwtVerifier.verify(token);/// 如果有问题的话，那么抛出各种异常

        System.out.println("TOKEN 验证通过");
        System.out.println("username :"+ decodedJWT.getClaim("username").asString());
        System.out.println("过期时间："+ decodedJWT.getExpiresAt());

    }
}
